///////////////////////////////////////////////////////////////////
//
// Apr/2018  M. Labiche -> DetectorConstruction.cc
//
///////////////////////////////////////////////////////////////////

#include "DetectorConstruction.hh"

#include "G4NistManager.hh"
#include "G4Material.hh"
#include "G4MaterialTable.hh"
#include "G4Element.hh"
#include "G4ElementTable.hh"
#include "G4Box.hh"
#include "G4Tubs.hh"
#include "G4SubtractionSolid.hh"
#include "G4Polycone.hh"
#include "G4LogicalVolume.hh"
#include "G4ThreeVector.hh"
#include "G4PVPlacement.hh"
#include "G4VisAttributes.hh"

#include "G4RunManager.hh"
#include "G4GeometryManager.hh"
#include "G4SolidStore.hh"
#include "G4LogicalVolumeStore.hh"
#include "G4PhysicalVolumeStore.hh"
#include "G4GenericMessenger.hh"

#include "G4GlobalMagFieldMessenger.hh"
#include "G4AutoDelete.hh"


using namespace CLHEP;

DetectorConstruction::DetectorConstruction()
:G4VUserDetectorConstruction(),
fMessenger(nullptr),
fsolidColliApert(nullptr),
fApertRadius(5.*cm) 
{ 

  fsolidColliApert= new G4Tubs("ColliApert",fApertRadius, 30.*cm,3.*cm,0.*deg,360.*deg);

  // define commands for this class
  DefineCommands();

}

DetectorConstruction::~DetectorConstruction()
{

	delete fMessenger;
	delete fsolidColliApert;

}

G4VPhysicalVolume* DetectorConstruction::Construct()
{
	
	//----------------------------------------------------
	// Material definitions
	//----------------------------------------------------
	
	
	   // This function illustrates the possible ways to define materials using
	   // G4 database on G4Elements   
	   G4NistManager* manager = G4NistManager::Instance();
	   manager->SetVerbose(0);
	   // Define Air material from NIST:

           G4Material* Air_NIST = manager->FindOrBuildMaterial("G4_AIR");
	   manager->ConstructNewGasMaterial("Air20","G4_AIR",293.*kelvin,1.*atmosphere);	

           G4Material* Lead_NIST = manager->FindOrBuildMaterial("G4_Pb");
	   manager->ConstructNewGasMaterial("Lead","G4_PB",293.*kelvin,1.*atmosphere);	
	   
           G4Material* SoftTissue = manager->FindOrBuildMaterial("G4_TISSUE_SOFT_ICRP");
	   manager->ConstructNewGasMaterial("SoftTissueICRP","G4_",293.*kelvin,1.*atmosphere);	
	


	
	// Define Material native Geant4
	
	G4String name, symbol;             //a=mass of a mole;
	G4double a, z, density;            //z=mean number of protons;  
	
	G4int ncomponents, natoms;         
	
	G4double pressure    = 3.e-18*pascal;
	G4double temperature = 2.73*kelvin;
	density     = 1.e-25*g/cm3;
	
	G4Material* Vacuum;
	Vacuum= new G4Material(name="Galactic", z=1., a=1.01*g/mole, 
       									density,kStateGas,temperature,pressure);
	
	
	//
	// define simple Elements
	//
	
	// N
	a = 14.01*g/mole;
	G4Element* N  = new G4Element(name="Nitrogen" ,symbol="N" , z= 7., a);

	// O
	a = 15.999*g/mole;
	G4Element* O  = new G4Element(name="Oxygen" ,symbol="O" , z= 8., a);
		
	// Mg
	a = 24.305*g/mole;
	G4Element* Mg  = new G4Element(name="Magnesium"  ,symbol="Mg" , z= 12., a);
	
	// Si
	a = 28.0855*g/mole;
	G4Element* Si  = new G4Element(name="Silicon"  ,symbol="Si" , z= 14., a);
	
	// Cl
	a = 35.453*g/mole;
	G4Element* Cl  = new G4Element(name="Chlorine"  ,symbol="Cl" , z= 17., a);
	
	// K
	a = 39.0983*g/mole;
	G4Element* K  = new G4Element(name="Potassium"  ,symbol="K" , z= 19., a);
	
	// Br
	a = 79.904*g/mole;
	G4Element* Br  = new G4Element(name="Bromine"  ,symbol="Br" , z= 35., a);
	
	// Sb
	a = 121.76*g/mole;
	G4Element* Sb = new G4Element(name="Antimony",symbol="Sb" , z= 51., a);
	
	// Cs
	a = 132.905*g/mole;
	G4Element* Cs = new G4Element(name="Cesium",symbol="Cs" , z= 55., a);
	
	// La
	a = 138.905*g/mole;
	G4Element* La = new G4Element(name="Lanthanum",symbol="La" , z= 57., a);

	// Na
	a = 22.98977*g/mole;
	G4Element* Na  = new G4Element(name="Sodium" ,symbol="Na" , z= 11., a);

	// I
	a = 126.9045*g/mole;
	G4Element* I  = new G4Element(name="Iodine" ,symbol="I" , z= 53., a);
	
	
	//
	// define simple materials
	//

 
    // NaI material
	density = 3.67*g/cm3;
	G4Material* NaImat = new G4Material(name="NaI", density, ncomponents=2);
	NaImat->AddElement(Na, natoms=1);
	NaImat->AddElement(I, natoms=1);

	// 12C material
	density = 2.26*g/cm3;
	a = 12.*g/mole;
	G4Material* Carbon = new G4Material(name="Carbon", z=6., a, density);

	
	// Air
	G4double fractionmass;
	density = 1.290*mg/cm3;
	G4Material* Air = new G4Material(name="Air", density, ncomponents=2);
	Air->AddElement(N, fractionmass=0.7);
	Air->AddElement(O, fractionmass=0.3);


	// Ge material
	density = 5.323*g/cm3;
	a = 72.59*g/mole;
	G4Material* Germanium = new G4Material(name="Germanium", z=32., a, density);

	// Al reflector
	density = 2.700*g/cm3;
	a = 26.98*g/mole;
	G4Material* AluR = new G4Material(name="AluRef", z=13., a, density);
	
	// MgO reflector
	density = 2.0*g/cm3;
	G4Material* MgO = new G4Material(name="MgO", density, ncomponents=2);
	MgO->AddElement(Mg, natoms=1);
	MgO->AddElement(O, natoms=1);
		
	// LaCl3
	density = 3.85*g/cm3;
	G4Material* LaCl3 = new G4Material(name="LaCl3", density, ncomponents=2);
	LaCl3->AddElement(Cl, natoms=3);
	LaCl3->AddElement(La, natoms=1);
	
	// LaBr3
	density = 5.08*g/cm3;
	G4Material* LaBr3 = new G4Material(name="LaBr3", density, ncomponents=2);
	LaBr3->AddElement(Br, natoms=3);
	LaBr3->AddElement(La, natoms=1);
	
	// Quartz (fused silica)
	density = 2.20*g/cm3;
	G4Material* Quartz = new G4Material(name="Quartz", density, ncomponents=2);
	Quartz->AddElement(Si, natoms=1);
	Quartz->AddElement(O, natoms=2);
	
	// Photocathode Material 
	// (Bialkali K2CsSb,  Density=?, Thickness=25.*nm?)
	density = 2.00*g/cm3;
	G4Material* K2CsSb = new G4Material(name="K2CsSb", density, ncomponents=3);
	K2CsSb->AddElement(K, natoms=2);
	K2CsSb->AddElement(Cs, natoms=1);
	K2CsSb->AddElement(Sb, natoms=1);
		
	//------------------------------------------------------
	// Detector geometry
	//------------------------------------------------------
	
	//     
	// World
	//
	
	G4double WorldSize= 50.*cm; 
	
	G4Box* solidWorld = new G4Box("World",		       	                  //its name
				       WorldSize/2,WorldSize/2,WorldSize/2);  //its size
	
	G4LogicalVolume* logicWorld = new G4LogicalVolume(solidWorld,      	//its solid
							  Air,	        //its material
							 "World");		    //its name
	
	G4VPhysicalVolume* physiWorld = new G4PVPlacement(0,			    //no rotation
				 			  G4ThreeVector(),	//at (0,0,0)
							  "World",		    //its name
							  logicWorld,		//its logical volume
							  NULL,		    //its mother  volume
							  false,	       	//no boolean operation
							  0);			    //copy number
	
	
	
	//
	// "Detector" mother volume 
	//
	
	G4double DetMothSize =3*cm;
	G4Box*   solidDetMoth = new G4Box("DetMoth",		       	                  //its name
					   DetMothSize/2,DetMothSize/2,DetMothSize/2);  //its size
	
	G4LogicalVolume* logicDetMoth = new G4LogicalVolume(solidDetMoth,      	//its solid
							    Vacuum,	        //its material
							   "DetMother");		    //its name

	G4ThreeVector posDetMoth = G4ThreeVector(0.*cm,0.*cm,11.5*cm);


	G4VPhysicalVolume* physiDetMoth;
	physiDetMoth = new G4PVPlacement(0,posDetMoth,"DetMother",logicDetMoth,
									    physiWorld,false,0);


        //
	//  Sensitive Cells 
	//

	G4double CellSize =1.*cm;
	G4Box*   solidCell = new G4Box("Cell",		       	                  //its name
					   CellSize/2,CellSize/2,CellSize/2);  //its size
	
	G4LogicalVolume* logicCell = new G4LogicalVolume(solidCell,      	//its solid
							    SoftTissue,	        //its material
							   "Cell");		//its name



	G4ThreeVector posCell;
	G4VPhysicalVolume* physiCell;
	G4int copy=0;
	for(G4int i=-1; i<2; i++)
	 {
		for(G4int j=-1; j<2; j++)
		 {
			for(G4int k=-1; k<2; k++)
	 		{
	 			posCell = G4ThreeVector(i*cm,j*cm,k*cm);


				physiCell = new G4PVPlacement(0,posCell,"Cell",logicCell,
									    physiDetMoth,false,copy);
				copy++;
			}
		}	
	}



	//
	// "Collimator frame" 
	//
	
	G4double ColliThick = 1*cm;
//	G4Box* solidColliFr = new G4Box("ColliFrame",		       	          //its name
//					   WorldSize/2,WorldSize/2,ColliThick/2);  //its size	

        //
	// Collimator hole
	//
	
	//G4double ApertRadius= 1.*cm;
	G4double StartPhi= 0.*deg;
	G4double DeltaPhi= 360.*deg;
	
	//G4Tubs* solidColliApert = new G4Tubs("ColliApert",0.*cm,ApertRadius,ColliThick,StartPhi,DeltaPhi);
	//fsolidColliApert = new G4Tubs("ColliApert",0.cm,fApertRadius,ColliThick,StartPhi,DeltaPhi);
	fsolidColliApert = new G4Tubs("ColliApert",fApertRadius, 30.*cm,ColliThick,StartPhi,DeltaPhi);
			
	//
	// Building the Collimator
	//
//	G4SubtractionSolid* solidColli = new G4SubtractionSolid("Colli",solidColliFr,fsolidColliApert, 0, G4ThreeVector(0.0,0.0,0.0));
	
	
	G4LogicalVolume* logicColli = new G4LogicalVolume(fsolidColliApert,      	//its solid
							    Lead_NIST,	        //its material
							    "Colli");		    //its name

	G4ThreeVector posColli = G4ThreeVector(0.*cm,0.*cm,6.5*cm);


        G4VPhysicalVolume* physiColli;
	physiColli = new G4PVPlacement(0,posColli,"Colli",logicColli,
				        physiWorld,false,0);
	
	
	
	//------------------------------------------------------
	// visualization attributes
	//------------------------------------------------------
	
	logicWorld->SetVisAttributes(G4VisAttributes::Invisible);
	
	//Green color for cells mother volume
	G4VisAttributes* Att1= new G4VisAttributes(G4Colour(0.0,1.0,0.0));
	Att1->SetForceWireframe(true);
	logicDetMoth->SetVisAttributes(Att1);
	
	//Yellow color for cells
	G4VisAttributes* Att2= new G4VisAttributes(G4Colour(1.0,1.0,0.0));
        Att2->SetForceWireframe(true);
	logicCell->SetVisAttributes(Att2);
	
	//Red color for Collimator
	G4VisAttributes* Att3= new G4VisAttributes(G4Colour(1.0,0.0,0.0));
        Att3->SetForceSolid(true);
	logicColli->SetVisAttributes(Att3);
	    
	    
	    
	    
	//
	// always return the physical World
	//
	
	return physiWorld;
}


//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
 
void DetectorConstruction::ConstructSDandField()
{
  // Sensitive detectors
  //.....

  // Create global magnetic field messenger.
  // Uniform magnetic field is then created automatically if
  // the field value is not zero.
  G4ThreeVector fieldValue = G4ThreeVector();
  fMagFieldMessenger = new G4GlobalMagFieldMessenger(fieldValue);
  fMagFieldMessenger->SetVerboseLevel(1);
  
  // Register the field messenger for deleting
  G4AutoDelete::Register(fMagFieldMessenger);
}


//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void DetectorConstruction::SetColliRadius(G4double val)
{
  if (!fsolidColliApert) {
      G4cerr << "Collimator has not yet been constructed." << G4endl;
      return;
  }
  
  fApertRadius = val;

  fsolidColliApert->SetInnerRadius(fApertRadius);
  
  
  //fsolidColli = new G4SubtractionSolid("Colli",solidColliFr,fsolidColliApert, 0, G4ThreeVector(0.0,0.0,0.0));
  //flogicColli->removeDaughter(fphysColli);
  //fphysColli= new G4PVPlacement(0,fposColli,"Colli",flogicColli,
  //			         fphysiWorld,false,0);


  //G4GeometryManager::GetInstance()->OpenGeometry();
  //G4PhysicalVolumeStore::GetInstance()->Clean();
  //G4LogicalVolumeStore::GetInstance()->Clean();
  //G4SolidStore::GetInstance()->Clean(); 

  // tell G4RunManager that we change the geometry
  G4RunManager::GetRunManager()->GeometryHasBeenModified();

}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......




void DetectorConstruction::DefineCommands()
{
  // Define /B5/detector command directory using generic messenger class
  fMessenger = new G4GenericMessenger(this, 
                                      "/NPMasterclass/Collimator/", 
                                      "geometry control");

  // ColliRadius command
  auto& ColliRadiusCmd
    = fMessenger->DeclareMethodWithUnit("ColliRadius","cm",
                                &DetectorConstruction::SetColliRadius, 
                                "Set radius of the Collimator.");
  ColliRadiusCmd.SetParameterName("radius", true);
//  ColliRadiusCmd.SetRange("radius>=0. && radius<3.");
  ColliRadiusCmd.SetDefaultValue("1.");
}




//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
 /*
void DetectorConstruction::SetDetectorMaterial(G4String materialName)
{
  //G4NistManager* nistManager = G4NistManager::Instance();


 	//
	// define simple Elements
	//
	
	G4String name,symbol;             //a=mass of a mole;
	G4double a, z, density;    //z=mean number of protons;  
	G4int natoms,ncomponents;

	
	// Na
	a = 14.01*g/mole;
	G4Element* Na  = new G4Element(name="Sodium" ,symbol="Na" , z= 7., a);

	// I
	a = 15.999*g/mole;
	G4Element* I  = new G4Element(name="Iodine" ,symbol="I" , z= 8., a);
 
    // NaI material
		density = 5.323*g/cm3;
	G4Material* NaImat = new G4Material(name="NaI", density, ncomponents=2);
	NaImat->AddElement(Na, natoms=1);
	NaImat->AddElement(I, natoms=1);


	if(materialName == "NaI")
	{
       fDetMaterial = NaImat;
	}else
	{
		G4cout << "Material not found ! " << G4endl;
	}

}
 */

///////////////////
// The Messenger
///////////////////
/*
#include "G4UIdirectory.hh"
#include "G4UIcmdWithAString.hh"
#include "G4UIcmdWithoutParameter.hh"
#include "G4UIcmdWithADouble.hh"
#include "G4UIcmdWith3Vector.hh"
#include "G4UIcmdWithAnInteger.hh"
#include "G4UIcmdWithABool.hh"

DetectorArrayMessenger::DetectorArrayMessenger(DetectorArray* pTarget,G4String name)
:myTarget(pTarget)
{ 
  const char *aLine;
  G4String commandName;
  G4String directoryName;
  
  directoryName = name + "/detector/";
  


  commandName = directoryName + "CellMaterial";
  aLine = commandName.c_str();
  DetMatCmd = new G4UIcmdWithAString(aLine, this);
  DetMatCmd->SetGuidance("Select Material of the detector.");
  DetMatCmd->SetGuidance("Required parameters: 1 string.");
  DetMatCmd->SetParameterName("choice",false);
  DetMatCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  commandName = directoryName + "ColliMaterial";
  aLine = commandName.c_str();
  WalMatCmd = new G4UIcmdWithAString(aLine, this);
  WalMatCmd->SetGuidance("Select Material of the Collimator.");
  WalMatCmd->SetGuidance("Required parameters: 1 string.");
  WalMatCmd->SetParameterName("choice",false);
  WalMatCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  commandName = directoryName + "rotateArray";
  aLine = commandName.c_str();
  RotateArrayCmd = new G4UIcmdWithAString(aLine, this);
  RotateArrayCmd->SetGuidance("Select rotation of the array.");
  RotateArrayCmd->SetGuidance("Required parameters: 2 double (rotation angles in degrees).");
  RotateArrayCmd->SetParameterName("choice",false);
  RotateArrayCmd->AvailableForStates(G4State_PreInit,G4State_Idle);


  commandName = directoryName + "traslateArray";
  aLine = commandName.c_str();
  TraslateArrayCmd = new G4UIcmdWith3Vector(aLine, this);
  TraslateArrayCmd->SetGuidance("Select traslation of the array.");
  TraslateArrayCmd->SetGuidance("Required parameters: 3 double (x, y, z components in mm).");
  TraslateArrayCmd->AvailableForStates(G4State_PreInit,G4State_Idle);


}*/
